#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <getopt.h>

#include "mcp3204.h"

#define MAIN_C
#include "exitfail.h"

#define BASENAME(p)   ((strrchr((p), '/') ? : ((p) - 1)) + 1)

#define REF_VOLTAGE	3.3	/* 基準電圧[V] */
#define DEFAULT_CH	0	/* オプションで指定されなかった場合に使用するチャンネル */

static void usage(const char *prog)
{
	printf("usage: %s <-d|--device FILENAME> [-c|--channel CHANNEL]\n", BASENAME(prog));
}

static void parse_arg(int argc, char *argv[], const char **device, int *ch)
{
	int c;
	char *endptr;

	*device = NULL;

	for(;;) {
		int option_index = 0;
		static struct option long_options[] = {
			/* name,        has_arg,           flag, val*/
			{"device",      required_argument, NULL, 'd'},
			{"channel",     required_argument, NULL, 'c'},
			{0,             0,                 0,    0},
		};

		c = getopt_long(argc, argv, "d:c:",
				long_options, &option_index);
		if (c == -1)
			break;

		switch (c) {
		case 'd':
			*device = optarg;
			break;
		case 'c':
			errno = 0;
			*ch = strtol(optarg, &endptr, 0);
			if (errno != 0 || optarg == endptr)
				goto err;

			if (*ch < MCP3204_CH_MIN || MCP3204_CH_MAX < *ch)
				goto err;
			break;
		default:
			goto err;
		}
	}

	if (*device == NULL)
		goto err;

	return;

err:
	usage(argv[0]);
	exit(EXIT_FAILURE);
}

int main(int argc, char *argv[])
{
	struct mcp3204 *adc;
	const char *device;
	int channel = DEFAULT_CH;
	double voltage;
	uint16_t digital_code;
	int ret;

	exitfail_init();

	parse_arg(argc, argv, &device, &channel);

	adc = mcp3204_open(device);
	if (adc == NULL)
		exitfail_errno("mcp3204_open");

	ret = mcp3204_read(adc, channel, &digital_code);
	if (ret != 0)
		exitfail_errno("mcp3204_read");

	voltage = digital_code * REF_VOLTAGE /
		((1 << MCP3204_RESOLUTION_BITS) - 1);

	printf("%1.3fV\n", voltage);

	ret = mcp3204_close(adc);
	if (ret != 0)
		exitfail_errno("mcp3204_close");

	return EXIT_SUCCESS;
}
